# Common makefile for Extensible Interpreter Design Kit using Borland C++

XIDEK = ..\..\..

SUPPORT_DIR = $(XIDEK)\support\$(SUPPORT_EXT)
LANG_DIR =    $(XIDEK)\$(LANG)\$(LANG_EXT)
DRIVER_DIR =  $(XIDEK)\driver
AGCLIB_DIR =  $(XIDEK)\agclib1
TEST_DIR =    $(XIDEK)\test\$(LANG)

!ifndef RES
RES = r2
!endif

CC = bcc32

CFLAGS  = -w-inl -w-hid -w-aus -w-par -I$(SUPPORT_DIR);$(LANG_DIR);$(AGCLIB_DIR)\include;$(%INCLUDE)

.AUTODEPEND

OBJS = demo.obj comdefs.obj bcidefs.obj astdefs.obj ast.obj\
       dxi.obj dci.obj astxi.obj astci.obj

SUPPORT_OBJS     = comdefs.obj bcidefs.obj astdefs.obj astxi.obj astci.obj
LANG_OBJS        = ast.obj dxi.obj dci.obj
DRIVER_OBJS      = demo.obj

DXI_OBJS   = demo.obj dxi.obj   comdefs.obj
DCI_OBJS   = demo.obj dci.obj   comdefs.obj bcidefs.obj
ASTXI_OBJS = demo.obj astxi.obj comdefs.obj ast.obj astdefs.obj
ASTCI_OBJS = demo.obj astci.obj comdefs.obj ast.obj astdefs.obj bcidefs.obj

AGCLIB = $(AGCLIB_DIR)\borland\agclib1.lib

PROGS  = dxi.exe dci.exe astxi.exe astci.exe

DX = t1.dx t2.dx t3.dx
DC = t1.dc t2.dc t3.dc
AX = t1.ax t2.ax t3.ax
AC = t1.ac t2.ac t3.ac

TESTS = $(DX) $(DC) $(AX) $(AC) $(EXT_TESTS)

# the first rule is the one you get when you type "make" by itself
all: progs tests

progs: $(PROGS)

tests: $(TESTS)

retest: clearTests $(TESTS)

dxi.exe: $(DXI_OBJS) $(AGCLIB)
	$(CC)  -e$@ $**

dci.exe: $(DCI_OBJS) $(AGCLIB)
	$(CC)  -e$@ $**

astxi.exe: $(ASTXI_OBJS) $(AGCLIB)
	$(CC) -e$@ $**

astci.exe: $(ASTCI_OBJS) $(AGCLIB)
	$(CC) -e$@ $**

$(AGCLIB)::
	cd $(AGCLIB_DIR)\borland
	make
	cd ..\..\borland\$(LANG)\$(SUPPORT_EXT)

# Compile phase

#Locate cpp files
.path.cpp=$(SUPPORT_DIR);$(LANG_DIR);$(DRIVER_DIR)

dxi.obj:   $(LANG_DIR)\dxi.cpp
dci.obj:   $(LANG_DIR)\dci.cpp
ast.obj:   $(LANG_DIR)\ast.cpp

#Rules for compiling
.cpp.obj:
	$(CC) @&&|
	  $(CFLAGS) -c $** -o$@
|

#Rules for building parsers
.suffixes: .syn

$(LANG_DIR)\dxi.cpp: $(LANG_DIR)\dxi.syn
$(LANG_DIR)\dxi.h:   $(LANG_DIR)\dxi.syn
$(LANG_DIR)\dci.cpp: $(LANG_DIR)\dci.syn
$(LANG_DIR)\dci.h:   $(LANG_DIR)\dci.syn
$(LANG_DIR)\ast.cpp: $(LANG_DIR)\ast.syn
$(LANG_DIR)\ast.h:   $(LANG_DIR)\ast.syn

.path.syn=$(LANG_DIR);

.syn.cpp:
	cd $(LANG_DIR)
	agcl $^&
	cd ..\..\borland\$(LANG)\$(SUPPORT_EXT)

.syn.h:
	cd $:
	agcl $^&
	cd ..\..\..

#Tests
.suffixes: .dx .dc .ax .ac .scf

#Dependencies
t1.dx: t1.scf  dxi.exe
t1.dc: t1.scf  dci.exe
t1.ax: t1.scf  astxi.exe
t1.ac: t1.scf  astci.exe

t2.dx: t2.scf  dxi.exe
t2.dc: t2.scf  dci.exe
t2.ax: t2.scf  astxi.exe
t2.ac: t2.scf  astci.exe

t3.dx: t3.scf  dxi.exe
t3.dc: t3.scf  dci.exe
t3.ax: t3.scf  astxi.exe
t3.ac: t3.scf  astci.exe

#Rules for running tests

.path.scf=$(TEST_DIR)

.scf.dx:
	dxi $< > $^&.dx
	fc $^&.dx $(TEST_DIR)\$^&.$(RES)

.scf.dc:
	dci $< > $^&.dc
	fc $^&.dc $(TEST_DIR)\$^&.$(RES)

.scf.ax:
	astxi $< > $^&.ax
	fc $^&.ax $(TEST_DIR)\$^&.$(RES)

.scf.ac:
	astci $< > $^&.ac
	fc $^&.ac $(TEST_DIR)\$^&.$(RES)

# Remove object files and library file for a fresh build
# clean is the standard target name for "remove objects".
clean:  clearTests
	del $(OBJS)
        del $(PROGS)

clearTests:
	del *.dx *.dc *.ax *.ac
